import os
import imp
import inspect


class InvalidTestError(Exception):
    pass


def is_test_valid(test_module):
    """Test if the test_module file is valid

    We only check for the transaction class, since the generic transaction
    parent will raise a NotImplemented exception if the run method is not present
    :return: True if the test is valid, else False
    :rtype: bool
    :raises: InvalidTestError
    """
    if not hasattr(test_module, "Transaction"):
        raise InvalidTestError("No transaction class found")
    getattr(test_module, "Transaction")
    return True


def load_module(path):
    """Load a single module based on a path on the system

    :param str path: the full path of the file to load as a module
    :return: the module imported
    :rtype: mixed
    :raises: ImportError, InvalidTestError
    """
    if not os.path.exists(path):
        raise ImportError("File does not exists: {}".format(path))
    try:
        module_name = inspect.getmodulename(os.path.basename(path))
        module = imp.load_source(module_name, path)
        if is_test_valid(module):
            return module
        raise InvalidTestError("Module not valid")
    except ImportError as e:
        print("Error importing the tests script {}\nError: {}".format(module_name, e))
        raise InvalidTestError(e)


def load_file(file_name):
    """Load a single file based on its full path as a module

    :param str filename: the full path of the file
    :return: the module loaded
    :rtype: mixed
    :raises: ImportError, InvalidTestError
    """
    if not os.path.exists(file_name):
        raise ImportError("File does not exists: {}".format(file_name))
    realpath = os.path.realpath(os.path.abspath(file_name))
    return load_module(realpath)


def load_all(path, exclude=None):
    """Load all python scripts on the given folder path and return a dict containing thread_num

    the returned dict will be returned in the form::

        {
            'module_name': module_reference
        }

    :param str path: the pass of the folder
    :param list exclude: the list of files to exclude from the import
    :return: a dict containing the modules
    :rtype: dict
    """
    if exclude is None:
        exclude = []
    modules = {}
    package = os.path.realpath(os.path.abspath(path))
    if not os.path.isdir(package):
        return None
    for script in os.listdir(package):
        if not script.endswith('.py'):
            continue
        basename = os.path.basename(script)
        if basename.startswith('__') or basename == "octapp.py":
            continue
        os.path.join(package, basename)
        print("Loading module {}".format(basename))
        module = load_module(os.path.join(package, basename))
        if module:
            modules[module.__name__] = module
    return modules



import os
import sys
import time
import json
from six.moves import queue
from threading import Thread

from oct.results.models import Result, Turret, set_database, db


class ResultsWriter(Thread):
    """This class will handle results and stats comming from the turrets

    :param output_dir: the output directory for the results
    :type output_dir: str
    """
    def __init__(self, output_dir, config, queue=None):
        Thread.__init__(self)
        self.output_dir = output_dir
        self.trans_count = 0
        self.timer_count = 0
        self.error_count = 0
        self.turret_name = 'Turret'
        self.results = []

        if queue:
            self.queue = queue

        try:
            os.makedirs(self.output_dir, 0o755)
        except OSError:
            sys.stderr.write("ERROR: Can not create output directory\n")
            sys.exit(1)

        set_database(self.output_dir + "results.sqlite", db, config)
        db.connect()
        db.create_tables([Result, Turret])

    def write_turret(self, datas):
        turret = Turret(name=datas['turret'], canons=datas['canons'], script=datas['script'], rampup=datas['rampup'],
                        uuid=datas['uuid'], status=datas['status'])
        turret.save()
        return turret

    def write_result(self, datas):
        self.trans_count += 1
        self.timer_count += len(datas['custom_timers'])
        if datas['error']:
            self.error_count += 1

        result = Result(error=datas['error'], scriptrun_time=datas['scriptrun_time'], elapsed=datas['elapsed'],
                        epoch=datas['epoch'],
                        custom_timers=json.dumps(datas['custom_timers']), turret_name=datas['turret_name'])
        result.save()

    def run(self):
        while True:
            try:
                elapsed, epoch, self.user_group_name, scriptrun_time, error, custom_timers = self.queue.get(False)
                datas = dict(elapsed=elapsed, epoch=epoch, turret_name=self.user_group_name,
                             scriptrun_time=scriptrun_time,
                             error=error, custom_timers=custom_timers)
                self.write_result(datas)
            except queue.Empty:
                time.sleep(.05)



from __future__ import print_function
from celery import Celery, group, current_task
from celery.utils.log import get_task_logger
from oct.core.test_loader import load_file
from multiprocessing.sharedctypes import Array
import threading
import time
import sys
import logging
import os

# This is the main app, update it inside your project if you need to
app = Celery('oct', backend=None, broker=None)

logger = get_task_logger(__name__)
buff = Array("i", range(2))
buff[0] = 0
buff[1] = 0


def main_loop(project_name, cmd_opts, config, output_dir):
    """
    Main run loop, will run all the tests.

    :param project_name: the name of the project
    :param cmd_opts: all parsed commands
    :return: None
    """
    script_prefix = os.path.join(cmd_opts.projects_dir, project_name, "test_scripts")
    script_prefix = os.path.normpath(script_prefix)

    user_groups = []
    threads = 0
    for i, ug_config in enumerate(config.user_group_configs):
        script_file = os.path.join(script_prefix, ug_config.script_file)
        ug = UserGroup(process_num=i, group_name=ug_config.name, thread_num=ug_config.num_threads,
                       script_file=script_file, output_dir=output_dir,
                       run_time=config.run_time, rampup=config.rampup, console_logging=config.console_logging)
        user_groups.append(ug)
        threads += ug_config.num_threads

    res = group(run_ug.s(ug) for ug in user_groups)()

    if config.console_logging:
        res.get(timeout=config.run_time + 5)
    else:
        print('\nuser_groups: {0}'.format(len(user_groups)), end='')
        print('\tthreads: {0}'.format(threads))

        st = 0
        info_str = 'time : {0}/{1} seconds transactions: {2} errors: {3}'
        while not res.ready():
            try:
                info = res.results[0].info
                if not info and not res.ready:
                    print(info_str.format(st, config.run_time, 0, 0), end='\r')
                elif info is not None:
                    print(info_str.format(st, config.run_time, info['trans'], info['errors']), end='\r')
                time.sleep(1)
                st += 1
            except KeyboardInterrupt:
                # purge all tasks in case of user interruption
                print('\nPurging all tasks...')
                app.control.purge()
                # revoke all tasks in case of KeyBoardInterrupt
                res.revoke(terminate=True)
                sys.exit(1)
            except EnvironmentError:
                if st <= 1:
                    # pass if we just start the test and the worker need a warmup
                    pass


class Agent(threading.Thread):
    """
    An object representing a agent for executing the script. Inherit from thread class and will be start within celery
     task

    :param process_num: the number of the process
    :param thread_num: the id of the thread
    :param start_time: the start time of the thread
    :param run_time: the total run_time of the test
    :param user_group_name: the name of the related user group
    :param script_module: the module containing the script to execute
    :param console_logging: boolean to know if we must log inside the console
    """
    def __init__(self, process_num, thread_num, start_time, run_time,
                 user_group_name, script_module, script_file, console_logging):
        threading.Thread.__init__(self)
        self.process_num = process_num
        self.thread_num = thread_num
        self.start_time = start_time
        self.run_time = run_time
        self.user_group_name = user_group_name
        self.script_module = script_module
        self.script_file = script_file
        self.console_logging = console_logging
        self.error_count = 0
        self.trans_count = 0

        # choose most accurate timer to use (time.clock has finer granularity
        # than time.time on windows, but shouldn't be used on other systems).
        if sys.platform.startswith('win'):
            self.default_timer = time.clock
        else:
            self.default_timer = time.time

    def run(self):
        elapsed = 0
        trans = self.script_module.Transaction()
        trans.custom_timers = {}

        # scripts have access to these vars, which can be useful for loading unique data
        trans.thread_num = self.thread_num
        trans.process_num = self.process_num

        while elapsed < self.run_time:
            error = ''
            start = self.default_timer()

            try:
                trans.run()
            except Exception as e:  # test runner catches all script exceptions here
                error = str(e).replace(',', '')

            finish = self.default_timer()

            scriptrun_time = finish - start
            elapsed = time.time() - self.start_time
            self.trans_count += 1

            # don't include cleaning time into reports
            trans.br.clean_session()

            # update shared buffer
            buff[0] += 1

            epoch = time.mktime(time.localtime())

            if error != '':
                # Convert line breaks to literal \n so the CSV will be readable.
                error = '\\n'.join(error.splitlines())
                self.error_count += 1
                buff[1] += 1
            logger.info('%.3f|%i|%s|%f|%s|%s' % (elapsed, epoch,
                                                 self.user_group_name,
                                                 scriptrun_time, error,
                                                 repr(trans.custom_timers).replace(',', '--')))
            if self.console_logging:
                print(('%.3f| %i| %s| %.3f| %s| %s' % (elapsed, epoch,
                                                       self.user_group_name, scriptrun_time,
                                                       error, repr(trans.custom_timers).replace(',', '--'))))


class UserGroup(object):
    """
    This class represent an user group
    Does not actually run anything

    :param process_num: the id of the process
    :param thread_num: the id of the thread
    :param script_file: the path of the tested script
    :param run_time: the time to run
    :param rampup: rampup property in seconds
    :param group_name: the name of the virtual user group
    :param queue: the queue for child threads
    """
    def __init__(self, **kwargs):
        self.start_time = time.time()
        self.process_num = kwargs.pop('process_num', None)
        self.thread_num = kwargs.pop('thread_num', None)
        self.script_file = kwargs.pop('script_file', None)
        self.run_time = kwargs.pop('run_time', None)
        self.rampup = kwargs.pop('rampup', None)
        self.group_name = kwargs.pop('group_name', None)
        self.output_dir = kwargs.pop('output_dir', None)
        self.console_logging = kwargs.pop('console_logging', None)


@app.task
def run_ug(user_group):
    """
    Main celery task for running tests

    :param user_group: User group object containing all options
    :return: None
    """
    buff[0] = 0
    buff[1] = 0
    script_module = load_file(user_group.script_file)
    threads = []
    fh = logging.FileHandler(user_group.output_dir + 'results.csv')
    logger.addHandler(fh)
    current_task.update_state(state='RUNNING', meta={'trans': buff[0], 'errors': buff[1]})
    for i in range(user_group.thread_num):
        space = float(user_group.rampup) / float(user_group.thread_num)
        if i > 0:
            time.sleep(space)
        agent_thread = Agent(user_group.process_num, i,
                             user_group.start_time, user_group.run_time,
                             user_group.group_name,
                             script_module, user_group.script_file,
                             user_group.console_logging)
        agent_thread.daemon = True
        threads.append(agent_thread)
        agent_thread.start()
    while len([t for t in threads if t.is_alive()]) > 0:
        current_task.update_state(state='RUNNING', meta={'trans': buff[0], 'errors': buff[1]})
        time.sleep(1)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
#  Copyright (c) 2010-2012 Corey Goldberg (corey@goldb.org)
#  License: GNU LGPLv3
#
#  This file is part of Multi-Mechanize | Performance Test Framework
#
from __future__ import print_function
import multiprocessing
import optparse
import os
# -- NOT-NEEDED: import Queue
import shutil
import sys
import time
from datetime import datetime
from oct.results.resultsoutput import output as output_results


import oct.multimechanize.core as core
import oct.results.resultswriter as resultswriter
import oct.multimechanize.progressbar as progressbar
from oct.multimechanize import __version__ as version
from oct.utilities.configuration import configure
from oct.core.hq import HightQuarter


def main():
    """
    Main function to run multimechanize benchmark/performance test.
    """

    usage = 'Usage: %prog <project name> [options]'
    parser = optparse.OptionParser(usage=usage, version=version)
    parser.add_option('-p', '--port', dest='port', type='int', help='rpc listener port')
    parser.add_option('-r', '--results', dest='results_dir', help='results directory to reprocess')
    parser.add_option('-b', '--bind-addr', dest='bind_addr', help='rpc bind address', default='localhost')
    parser.add_option('-d', '--directory', dest='projects_dir', help='directory containing project folder', default='.')
    cmd_opts, args = parser.parse_args()

    try:
        project_name = args[0]
    except IndexError:
        sys.stderr.write('\nERROR: no project specified\n\n')
        sys.stderr.write('%s\n' % usage)
        sys.stderr.write('Example: multimech-run my_project\n\n')
        sys.exit(1)

    core.init(cmd_opts.projects_dir, project_name)

    # -- ORIGINAL-MAIN:
    if cmd_opts.results_dir:  # don't run a test, just re-process results
        rerun_results(project_name, cmd_opts, cmd_opts.results_dir)
    else:
        run(project_name, cmd_opts)
    return


def run(project_name, cmd_opts, remote_starter=None):
    if remote_starter is not None:
        remote_starter.test_running = True
        remote_starter.output_dir = None

    config = configure(project_name, cmd_opts)

    run_localtime = time.localtime()
    milisec = datetime.now().microsecond
    output_dir = '%s/%s/results/results_%s' % (cmd_opts.projects_dir, project_name,
                                               time.strftime('%Y.%m.%d_%H.%M.%S_'+str(milisec)+'/', run_localtime))

    # this queue is shared between all processes/threads
    rw = resultswriter.ResultsWriter(output_dir, config)

    script_prefix = os.path.join(cmd_opts.projects_dir, project_name, "test_scripts")
    script_prefix = os.path.normpath(script_prefix)

    hq = HightQuarter(config.get('publish_port', 5000), config.get('rc_port', 5001), rw, config)
    hq.wait_turrets(config.get("min_turrets", 1))
    hq.run()

    # all agents are done running at this point
    time.sleep(.2)  # make sure the writer queue is flushed
    print('\n\nanalyzing results...\n')
    if output_results(output_dir, 'results.sqlite', config):
        print('created: %sresults.html\n' % output_dir)

    # copy config file to results directory
    project_config = os.sep.join([cmd_opts.projects_dir, project_name, 'config.json'])
    saved_config = os.sep.join([output_dir, 'config.json'])
    shutil.copy(project_config, saved_config)

    print('done.\n')

    if remote_starter is not None:
        remote_starter.test_running = False
        remote_starter.output_dir = output_dir

    return


def rerun_results(project_name, cmd_opts, results_dir):
    output_dir = '%s/%s/results/%s/' % (cmd_opts.projects_dir, project_name, results_dir)
    saved_config = '%s/config.cfg' % output_dir
    (run_time, rampup, results_ts_interval, console_logging, progress_bar, results_database,
     post_run_script, xml_report, user_group_configs) = configure(project_name, cmd_opts, config_file=saved_config)
    print('\n\nanalyzing results...\n')
    # results.output_results(output_dir, 'results.csv', run_time, rampup, results_ts_interval, user_group_configs,
    #                       xml_report)
    print('created: %sresults.html\n' % output_dir)
    if xml_report:
        print('created: %sresults.jtl' % output_dir)
        print('created: last_results.jtl\n')


class UserGroupConfig(object):
    def __init__(self, num_threads, name, script_file):
        self.num_threads = num_threads
        self.name = name
        self.script_file = script_file


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
#  Copyright (c) 2010 Corey Goldberg (corey@goldb.org)
#  License: GNU LGPLv3
#
#  This file is part of Multi-Mechanize | Performance Test Framework
#


""" script to verify all multi-mechanize dependencies are satisfied """


import sys


if sys.version_info >= (3,):
    print('sorry, no py3k support yet')
elif sys.version_info < (2, 6, 3):
    print('incompatible python version detected: %s.  Minimum version supported is 2.6' % repr(sys.version_info))
else:
    print('compatible python version detected: %s' % repr(sys.version_info))

try:
    import sqlalchemy
    print('imported SQLAlchemy succesfully')
except ImportError:
    print('can not import SQLAlchemy')




import os
from setuptools import setup
from oct import __version__

BASE_DIR = os.path.abspath(os.path.dirname(__file__))

with open('README.rst') as f:
    long_description = f.read()

setup(
    name='oct',
    version=__version__,
    author='Emmanuel Valette',
    author_email='manu.valette@gmail.com',
    packages=['oct', 'oct.core', 'oct.multimechanize', 'oct.testing',
              'oct.multimechanize.utilities', 'oct.utilities', 'oct.tools', 'oct.results'],
    package_data={
        'oct.utilities': ['templates/css/*', 'templates/configuration/*', 'templates/html/*', 'templates/scripts/*']
    },
    description="A library for performances testing, give you the tools for load testing anything",
    long_description=long_description,
    url='https://github.com/karec/oct',
    download_url='https://github.com/karec/oct/archive/master.zip',
    keywords=['testing', 'perfs', 'performances', 'web', 'load', 'zeromq'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4'
    ],
    install_requires=[
        'argparse',
        'pygal',
        'peewee',
        'oct-turrets',
        'six',
        'pyzmq',
        'numpy',
        'jinja2'
    ],
    entry_points={'console_scripts': [
        'multimech-run = oct.multimechanize.utilities.run:main',
        'multimech-newproject = oct.multimechanize.utilities.newproject:main',
        'multimech-gridgui = oct.multimechanize.utilities.gridgui:main',
        'oct-run = oct.utilities.run:main',
        'oct-newproject = oct.utilities.newproject:main',
        'octtools-sitemap-to-csv = oct.tools.xmltocsv:main',
        'octtools-user-generator = oct.tools.email_generator:email_generator',
        'oct-tocsv = oct.tools.results_to_csv:main',
        'oct-rebuild-results = oct.tools.rebuild_results:main',
        'oct-pack-turrets = oct.utilities.pack:main'
    ]},
)
